@@ -23,6 +23,7 @@ from osc_lib import utils
 
 from openstackclient.i18n import _
 from openstackclient.identity import common as identity_common
+from openstackclient.network import sdk_utils
 
 
 LOG = logging.getLogger(__name__)
@@ -84,6 +85,15 @@ def _get_common_parse_arguments(parser, is_create=True):
         help=_("DNS server for this subnet "
                "(repeat option to set multiple DNS servers)")
     )
+
+    if not is_create:
+        parser.add_argument(
+            '--no-dns-nameservers',
+            action='store_true',
+            help=_("Clear existing information of DNS Nameservers. "
+                   "Specify both --dns-nameserver and --no-dns-nameserver "
+                   "to overwrite the current DNS Nameserver information.")
+        )
     parser.add_argument(
         '--host-route',
         metavar='destination=<subnet>,gateway=<ip-address>',
@@ -117,11 +127,12 @@ def _get_common_parse_arguments(parser, is_create=True):
 
 
 def _get_columns(item):
-    columns = list(item.keys())
-    if 'tenant_id' in columns:
-        columns.remove('tenant_id')
-        columns.append('project_id')
-    return tuple(sorted(columns))
+    column_map = {
+        'is_dhcp_enabled': 'enable_dhcp',
+        'subnet_pool_id': 'subnetpool_id',
+        'tenant_id': 'project_id',
+    }
+    return sdk_utils.get_osc_show_columns_for_sdk_resource(item, column_map)
 
 
 def convert_entries_to_nexthop(entries):
@@ -217,8 +228,10 @@ def _get_attrs(client_manager, parsed_args, is_create=True):
     return attrs
 
 
+# TODO(abhiraut): Use the SDK resource mapped attribute names once the
+# OSC minimum requirements include SDK 1.0.
 class CreateSubnet(command.ShowOne):
-    """Create a subnet"""
+    _description = _("Create a subnet")
 
     def get_parser(self, prog_name):
         parser = super(CreateSubnet, self).get_parser(prog_name)
@@ -323,13 +336,13 @@ class CreateSubnet(command.ShowOne):
         client = self.app.client_manager.network
         attrs = _get_attrs(self.app.client_manager, parsed_args)
         obj = client.create_subnet(**attrs)
-        columns = _get_columns(obj)
+        display_columns, columns = _get_columns(obj)
         data = utils.get_item_properties(obj, columns, formatters=_formatters)
-        return (columns, data)
+        return (display_columns, data)
 
 
 class DeleteSubnet(command.Command):
-    """Delete subnet(s)"""
+    _description = _("Delete subnet(s)")
 
     def get_parser(self, prog_name):
         parser = super(DeleteSubnet, self).get_parser(prog_name)
@@ -362,8 +375,10 @@ class DeleteSubnet(command.Command):
             raise exceptions.CommandError(msg)
 
 
+# TODO(abhiraut): Use only the SDK resource mapped attribute names once the
+# OSC minimum requirements include SDK 1.0.
 class ListSubnet(command.Lister):
-    """List subnets"""
+    _description = _("List subnets")
 
     def get_parser(self, prog_name):
         parser = super(ListSubnet, self).get_parser(prog_name)
@@ -443,8 +458,10 @@ class ListSubnet(command.Lister):
             filters['ip_version'] = parsed_args.ip_version
         if parsed_args.dhcp:
             filters['enable_dhcp'] = True
+            filters['is_dhcp_enabled'] = True
         elif parsed_args.no_dhcp:
             filters['enable_dhcp'] = False
+            filters['is_dhcp_enabled'] = False
         if parsed_args.service_types:
             filters['service_types'] = parsed_args.service_types
         if parsed_args.project:
@@ -454,6 +471,7 @@ class ListSubnet(command.Lister):
                 parsed_args.project_domain,
             ).id
             filters['tenant_id'] = project_id
+            filters['project_id'] = project_id
         if parsed_args.network:
             network_id = network_client.find_network(parsed_args.network,
                                                      ignore_missing=False).id
@@ -472,7 +490,7 @@ class ListSubnet(command.Lister):
             headers += ('Project', 'DHCP', 'Name Servers',
                         'Allocation Pools', 'Host Routes', 'IP Version',
                         'Gateway', 'Service Types')
-            columns += ('tenant_id', 'enable_dhcp', 'dns_nameservers',
+            columns += ('project_id', 'is_dhcp_enabled', 'dns_nameservers',
                         'allocation_pools', 'host_routes', 'ip_version',
                         'gateway_ip', 'service_types')
 
@@ -483,8 +501,10 @@ class ListSubnet(command.Lister):
                 ) for s in data))
 
 
+# TODO(abhiraut): Use the SDK resource mapped attribute names once the
+# OSC minimum requirements include SDK 1.0.
 class SetSubnet(command.Command):
-    """Set subnet properties"""
+    _description = _("Set subnet properties")
 
     def get_parser(self, prog_name):
         parser = super(SetSubnet, self).get_parser(prog_name)
@@ -532,7 +552,10 @@ class SetSubnet(command.Command):
         attrs = _get_attrs(self.app.client_manager, parsed_args,
                            is_create=False)
         if 'dns_nameservers' in attrs:
-            attrs['dns_nameservers'] += obj.dns_nameservers
+            if not parsed_args.no_dns_nameservers:
+                attrs['dns_nameservers'] += obj.dns_nameservers
+        elif parsed_args.no_dns_nameservers:
+            attrs['dns_nameservers'] = []
         if 'host_routes' in attrs:
             if not parsed_args.no_host_route:
                 attrs['host_routes'] += obj.host_routes
@@ -542,7 +565,7 @@ class SetSubnet(command.Command):
             if not parsed_args.no_allocation_pool:
                 attrs['allocation_pools'] += obj.allocation_pools
         elif parsed_args.no_allocation_pool:
-            attrs['allocation_pools'] = ''
+            attrs['allocation_pools'] = []
         if 'service_types' in attrs:
             attrs['service_types'] += obj.service_types
         client.update_subnet(obj, **attrs)
@@ -550,7 +573,7 @@ class SetSubnet(command.Command):
 
 
 class ShowSubnet(command.ShowOne):
-    """Display subnet details"""
+    _description = _("Display subnet details")
 
     def get_parser(self, prog_name):
         parser = super(ShowSubnet, self).get_parser(prog_name)
@@ -564,13 +587,13 @@ class ShowSubnet(command.ShowOne):
     def take_action(self, parsed_args):
         obj = self.app.client_manager.network.find_subnet(parsed_args.subnet,
                                                           ignore_missing=False)
-        columns = _get_columns(obj)
+        display_columns, columns = _get_columns(obj)
         data = utils.get_item_properties(obj, columns, formatters=_formatters)
-        return (columns, data)
+        return (display_columns, data)
 
 
 class UnsetSubnet(command.Command):
-    """Unset subnet properties"""
+    _description = _("Unset subnet properties")
 
     def get_parser(self, prog_name):
         parser = super(UnsetSubnet, self).get_parser(prog_name)
