@@ -148,10 +148,10 @@ class OpenStackShell(shell.OpenStackShell):
                     'auth_type': self._auth_type,
                 },
             )
-        except (IOError, OSError) as e:
+        except (IOError, OSError):
             self.log.critical("Could not read clouds.yaml configuration file")
             self.print_help_if_requested()
-            raise e
+            raise
 
         if not self.options.debug:
             self.options.debug = None
