@@ -194,15 +194,18 @@ class FakeIPAvailability(object):
     """Fake one or more network ip availabilities."""
 
     @staticmethod
-    def create_one_ip_availability():
+    def create_one_ip_availability(attrs=None):
         """Create a fake list with ip availability stats of a network.
 
+        :param Dictionary attrs:
+            A dictionary with all attributes
         :return:
             A FakeResource object with network_name, network_id, etc.
         """
+        attrs = attrs or {}
 
         # Set default attributes.
-        network_ip_availability = {
+        network_ip_attrs = {
             'network_id': 'network-id-' + uuid.uuid4().hex,
             'network_name': 'network-name-' + uuid.uuid4().hex,
             'tenant_id': '',
@@ -210,10 +213,13 @@ class FakeIPAvailability(object):
             'total_ips': 254,
             'used_ips': 6,
         }
+        network_ip_attrs.update(attrs)
 
         network_ip_availability = fakes.FakeResource(
-            info=copy.deepcopy(network_ip_availability),
+            info=copy.deepcopy(network_ip_attrs),
             loaded=True)
+        network_ip_availability.project_id = network_ip_attrs['tenant_id']
+
         return network_ip_availability
 
     @staticmethod
@@ -291,14 +297,17 @@ class FakeNetwork(object):
             'admin_state_up': True,
             'shared': False,
             'subnets': ['a', 'b'],
-            'provider_network_type': 'vlan',
-            'provider_physical_network': 'physnet1',
-            'provider_segmentation_id': "400",
+            'provider:network_type': 'vlan',
+            'provider:physical_network': 'physnet1',
+            'provider:segmentation_id': "400",
             'router:external': True,
             'availability_zones': [],
             'availability_zone_hints': [],
             'is_default': False,
             'port_security_enabled': True,
+            'qos_policy_id': 'qos-policy-id-' + uuid.uuid4().hex,
+            'ipv4_address_scope': 'ipv4' + uuid.uuid4().hex,
+            'ipv6_address_scope': 'ipv6' + uuid.uuid4().hex,
         }
 
         # Overwrite default attributes.
@@ -310,8 +319,21 @@ class FakeNetwork(object):
         # Set attributes with special mapping in OpenStack SDK.
         network.project_id = network_attrs['tenant_id']
         network.is_router_external = network_attrs['router:external']
+        network.is_admin_state_up = network_attrs['admin_state_up']
         network.is_port_security_enabled = \
             network_attrs['port_security_enabled']
+        network.subnet_ids = network_attrs['subnets']
+        network.is_shared = network_attrs['shared']
+        network.provider_network_type = \
+            network_attrs['provider:network_type']
+        network.provider_physical_network = \
+            network_attrs['provider:physical_network']
+        network.provider_segmentation_id = \
+            network_attrs['provider:segmentation_id']
+        network.ipv4_address_scope_id = \
+            network_attrs['ipv4_address_scope']
+        network.ipv6_address_scope_id = \
+            network_attrs['ipv6_address_scope']
 
         return network
 
@@ -454,12 +476,15 @@ class FakePort(object):
                                   loaded=True)
 
         # Set attributes with special mappings in OpenStack SDK.
-        port.project_id = port_attrs['tenant_id']
         port.binding_host_id = port_attrs['binding:host_id']
         port.binding_profile = port_attrs['binding:profile']
         port.binding_vif_details = port_attrs['binding:vif_details']
         port.binding_vif_type = port_attrs['binding:vif_type']
         port.binding_vnic_type = port_attrs['binding:vnic_type']
+        port.is_admin_state_up = port_attrs['admin_state_up']
+        port.is_port_security_enabled = port_attrs['port_security_enabled']
+        port.project_id = port_attrs['tenant_id']
+        port.security_group_ids = port_attrs['security_groups']
 
         return port
 
@@ -745,6 +770,7 @@ class FakeNetworkQosPolicy(object):
             loaded=True)
 
         # Set attributes with special mapping in OpenStack SDK.
+        qos_policy.is_shared = qos_policy_attrs['shared']
         qos_policy.project_id = qos_policy_attrs['tenant_id']
 
         return qos_policy
@@ -826,6 +852,9 @@ class FakeRouter(object):
 
         # Set attributes with special mapping in OpenStack SDK.
         router.project_id = router_attrs['tenant_id']
+        router.is_admin_state_up = router_attrs['admin_state_up']
+        router.is_distributed = router_attrs['distributed']
+        router.is_ha = router_attrs['ha']
 
         return router
 
@@ -1063,6 +1092,8 @@ class FakeSubnet(object):
                                     loaded=True)
 
         # Set attributes with special mappings in OpenStack SDK.
+        subnet.is_dhcp_enabled = subnet_attrs['enable_dhcp']
+        subnet.subnet_pool_id = subnet_attrs['subnetpool_id']
         subnet.project_id = subnet_attrs['tenant_id']
 
         return subnet
@@ -1268,3 +1299,38 @@ class FakeSubnetPool(object):
         if subnet_pools is None:
             subnet_pools = FakeSubnetPool.create_subnet_pools(count)
         return mock.Mock(side_effect=subnet_pools)
+
+
+class FakeNetworkServiceProvider(object):
+    """Fake Network Service Providers"""
+
+    @staticmethod
+    def create_one_network_service_provider(attrs=None):
+        """Create service provider"""
+        attrs = attrs or {}
+
+        service_provider = {
+            'name': 'provider-name-' + uuid.uuid4().hex,
+            'service_type': 'service-type-' + uuid.uuid4().hex,
+            'default': False,
+        }
+
+        service_provider.update(attrs)
+
+        provider = fakes.FakeResource(
+            info=copy.deepcopy(service_provider),
+            loaded=True)
+        provider.is_default = service_provider['default']
+
+        return provider
+
+    @staticmethod
+    def create_network_service_providers(attrs=None, count=2):
+        """Create multiple service providers"""
+
+        service_providers = []
+        for i in range(0, count):
+            service_providers.append(FakeNetworkServiceProvider.
+                                     create_one_network_service_provider(
+                                         attrs))
+        return service_providers
