@@ -45,7 +45,7 @@ def _check_size_arg(args):
 
 
 class CreateVolume(command.ShowOne):
-    """Create new volume"""
+    _description = _("Create new volume")
 
     def get_parser(self, prog_name):
         parser = super(CreateVolume, self).get_parser(prog_name)
@@ -132,6 +132,28 @@ class CreateVolume(command.ShowOne):
             help=_("Allow volume to be attached more than once "
                    "(default to False)")
         )
+        bootable_group = parser.add_mutually_exclusive_group()
+        bootable_group.add_argument(
+            "--bootable",
+            action="store_true",
+            help=_("Mark volume as bootable")
+        )
+        bootable_group.add_argument(
+            "--non-bootable",
+            action="store_true",
+            help=_("Mark volume as non-bootable (default)")
+        )
+        readonly_group = parser.add_mutually_exclusive_group()
+        readonly_group.add_argument(
+            "--read-only",
+            action="store_true",
+            help=_("Set volume to read-only access mode")
+        )
+        readonly_group.add_argument(
+            "--read-write",
+            action="store_true",
+            help=_("Set volume to read-write access mode (default)")
+        )
         return parser
 
     def take_action(self, parsed_args):
@@ -199,6 +221,22 @@ class CreateVolume(command.ShowOne):
             multiattach=parsed_args.multi_attach,
             scheduler_hints=parsed_args.hint,
         )
+
+        if parsed_args.bootable or parsed_args.non_bootable:
+            try:
+                volume_client.volumes.set_bootable(
+                    volume.id, parsed_args.bootable)
+            except Exception as e:
+                LOG.error(_("Failed to set volume bootable property: %s"), e)
+        if parsed_args.read_only or parsed_args.read_write:
+            try:
+                volume_client.volumes.update_readonly_flag(
+                    volume.id,
+                    parsed_args.read_only)
+            except Exception as e:
+                LOG.error(_("Failed to set volume read-only access "
+                            "mode flag: %s"), e)
+
         # Remove key links from being displayed
         volume._info.update(
             {
@@ -211,7 +249,7 @@ class CreateVolume(command.ShowOne):
 
 
 class DeleteVolume(command.Command):
-    """Delete volume(s)"""
+    _description = _("Delete volume(s)")
 
     def get_parser(self, prog_name):
         parser = super(DeleteVolume, self).get_parser(prog_name)
@@ -263,7 +301,7 @@ class DeleteVolume(command.Command):
 
 
 class ListVolume(command.Lister):
-    """List volumes"""
+    _description = _("List volumes")
 
     def get_parser(self, prog_name):
         parser = super(ListVolume, self).get_parser(prog_name)
@@ -409,8 +447,55 @@ class ListVolume(command.Lister):
                 ) for s in data))
 
 
+class MigrateVolume(command.Command):
+    _description = _("Migrate volume to a new host")
+
+    def get_parser(self, prog_name):
+        parser = super(MigrateVolume, self).get_parser(prog_name)
+        parser.add_argument(
+            'volume',
+            metavar="<volume>",
+            help=_("Volume to migrate (name or ID)")
+        )
+        parser.add_argument(
+            '--host',
+            metavar="<host>",
+            required=True,
+            help=_("Destination host (takes the form: host@backend-name#pool)")
+        )
+        parser.add_argument(
+            '--force-host-copy',
+            action="store_true",
+            help=_("Enable generic host-based force-migration, "
+                   "which bypasses driver optimizations")
+        )
+        lock_group = parser.add_mutually_exclusive_group()
+        lock_group.add_argument(
+            '--lock-volume',
+            action="store_true",
+            help=_("If specified, the volume state will be locked "
+                   "and will not allow a migration to be aborted "
+                   "(possibly by another operation)")
+        )
+        lock_group.add_argument(
+            '--unlock-volume',
+            action="store_true",
+            help=_("If specified, the volume state will not be "
+                   "locked and the a migration can be aborted "
+                   "(default) (possibly by another operation)")
+        )
+        return parser
+
+    def take_action(self, parsed_args):
+        volume_client = self.app.client_manager.volume
+        volume = utils.find_resource(volume_client.volumes, parsed_args.volume)
+        volume_client.volumes.migrate_volume(volume.id, parsed_args.host,
+                                             parsed_args.force_host_copy,
+                                             parsed_args.lock_volume,)
+
+
 class SetVolume(command.Command):
-    """Set volume properties"""
+    _description = _("Set volume properties")
 
     def get_parser(self, prog_name):
         parser = super(SetVolume, self).get_parser(prog_name)
@@ -462,6 +547,19 @@ class SetVolume(command.Command):
                    'in the database with no regard to actual status, '
                    'exercise caution when using)'),
         )
+        parser.add_argument(
+            '--type',
+            metavar='<volume-type>',
+            help=_('New volume type (name or ID)'),
+        )
+        parser.add_argument(
+            '--retype-policy',
+            metavar='<retype-policy>',
+            choices=['never', 'on-demand'],
+            help=_('Migration policy while re-typing volume '
+                   '("never" or "on-demand", default is "never" ) '
+                   '(available only when "--type" option is specified)'),
+        )
         bootable_group = parser.add_mutually_exclusive_group()
         bootable_group.add_argument(
             "--bootable",
@@ -473,6 +571,17 @@ class SetVolume(command.Command):
             action="store_true",
             help=_("Mark volume as non-bootable")
         )
+        readonly_group = parser.add_mutually_exclusive_group()
+        readonly_group.add_argument(
+            "--read-only",
+            action="store_true",
+            help=_("Set volume to read-only access mode")
+        )
+        readonly_group.add_argument(
+            "--read-write",
+            action="store_true",
+            help=_("Set volume to read-write access mode")
+        )
         return parser
 
     def take_action(self, parsed_args):
@@ -484,10 +593,11 @@ class SetVolume(command.Command):
             try:
                 if volume.status != 'available':
                     msg = (_("Volume is in %s state, it must be available "
-                           "before size can be extended"), volume.status)
+                           "before size can be extended") % volume.status)
                     raise exceptions.CommandError(msg)
                 if parsed_args.size <= volume.size:
-                    msg = _("New size must be greater than %s GB"), volume.size
+                    msg = (_("New size must be greater than %s GB")
+                           % volume.size)
                     raise exceptions.CommandError(msg)
                 volume_client.volumes.extend(volume.id, parsed_args.size)
             except Exception as e:
@@ -522,6 +632,37 @@ class SetVolume(command.Command):
             except Exception as e:
                 LOG.error(_("Failed to set volume bootable property: %s"), e)
                 result += 1
+        if parsed_args.read_only or parsed_args.read_write:
+            try:
+                volume_client.volumes.update_readonly_flag(
+                    volume.id,
+                    parsed_args.read_only)
+            except Exception as e:
+                LOG.error(_("Failed to set volume read-only access "
+                            "mode flag: %s"), e)
+                result += 1
+        if parsed_args.type:
+            # get the migration policy
+            migration_policy = 'never'
+            if parsed_args.retype_policy:
+                migration_policy = parsed_args.retype_policy
+            try:
+                # find the volume type
+                volume_type = utils.find_resource(
+                    volume_client.volume_types,
+                    parsed_args.type)
+                # reset to the new volume type
+                volume_client.volumes.retype(
+                    volume.id,
+                    volume_type.id,
+                    migration_policy)
+            except Exception as e:
+                LOG.error(_("Failed to set volume type: %s"), e)
+                result += 1
+        elif parsed_args.retype_policy:
+            # If the "--retype-policy" is specified without "--type"
+            LOG.warning(_("'--retype-policy' option will not work "
+                          "without '--type' option"))
 
         kwargs = {}
         if parsed_args.name:
@@ -542,7 +683,7 @@ class SetVolume(command.Command):
 
 
 class ShowVolume(command.ShowOne):
-    """Display volume details"""
+    _description = _("Display volume details")
 
     def get_parser(self, prog_name):
         parser = super(ShowVolume, self).get_parser(prog_name)
@@ -573,7 +714,7 @@ class ShowVolume(command.ShowOne):
 
 
 class UnsetVolume(command.Command):
-    """Unset volume properties"""
+    _description = _("Unset volume properties")
 
     def get_parser(self, prog_name):
         parser = super(UnsetVolume, self).get_parser(prog_name)
